/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Daniel A. Reed (reed@cs.uiuc.edu)
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */

/*
 * FormDialogWidget.cc:
 *
 * $Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/Motif/PrimitiveWidgets/RCS/FormDialogWidget.C,v 1.10 1994/02/25 04:43:48 aydt Exp $
 *
 */



/*
$Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/Motif/PrimitiveWidgets/RCS/FormDialogWidget.C,v 1.10 1994/02/25 04:43:48 aydt Exp $
*/

#include "FormDialogWidget.h"

#include <X11/Xlib.h>
#include <Xm/Form.h>

#include "XArgs.h"
#include "util.h"

FormDialogWidget::FormDialogWidget(WidgetBase *par,
						     const XArgs &xargs,
						     const CString &name)
			: WidgetBase(par)
{
    CString uniqueName;

    if (name.length() == 0)
	    uniqueName = getUniqueName("FormDialog");
    else
	    uniqueName = name;

    _setType("FormDialogWidget");
    
    widget = XmCreateFormDialog(par->getWidget(),
				(char *)uniqueName.getValue(),
				xargs.getArgs(), xargs.getNArgs());
}

FormDialogWidget::~FormDialogWidget()
{
}

void		/* virtual */
FormDialogWidget::manageFormDialog()
{
	/* 
	 * Restore our saved position and dimension.
	 * For some reason I can't understand, it seems necessary to 
	 * call XtSetValues both before and after the widget is managed
	 * in order for them to work. Even then, the first time you
	 * call this under twm, the X coordinate isn't restored properly.
	 * It seems to work under mwm... <sigh>.  I do the MapWidget
	 * explicitly after the second XtSetValues to try an eliminate
	 * flicker.
	 */

        Arg	arglist[4];

        XtSetArg( arglist[0], XmNx, (XtArgVal) saveX );
        XtSetArg( arglist[1], XmNy, (XtArgVal) saveY );
        XtSetArg( arglist[2], XmNwidth, (XtArgVal) saveWidth );
        XtSetArg( arglist[3], XmNheight, (XtArgVal) saveHeight );

	XtSetValues( widget, arglist, 4 );

	XtSetMappedWhenManaged( widget, False );
	XtManageChild( widget );
	
	XtSetValues( widget, arglist, 4 );
	XtMapWidget( widget );
	XtSetMappedWhenManaged( widget, True );
}

void		/* virtual */
FormDialogWidget::unmanageFormDialog()
{
	/* 
	 * This method gets the position and dimensions of the dialog widget
	 * and saves them (adjusted to allow for window manager decorations)
	 * so that they can be restored after the dialog is remanaged.
	 */

	Arg 		arglist[4];
	Position	x, y;
	Dimension	width, height;

	XtSetArg( arglist[0], XmNx, (XtArgVal) &x );
	XtSetArg( arglist[1], XmNy, (XtArgVal) &y );
	XtSetArg( arglist[2], XmNwidth, (XtArgVal) &width );
	XtSetArg( arglist[3], XmNheight, (XtArgVal) &height );
	XtGetValues( widget, arglist, 4 );

	Widget  parentWidget = getParent()->getWidget();
	XWindowAttributes parentAttrs;
	XGetWindowAttributes( XtDisplay( parentWidget ),
			      XtWindow( parentWidget ), 
			      &parentAttrs );

	saveX = (int) x - parentAttrs.x;
	saveY = (int) y - parentAttrs.y;
	saveWidth = (int) width;
	saveHeight = (int) height;

	XtUnmanageChild( widget );
}
